@@ -19,17 +19,21 @@ DATASET_LOGLEVEL_DETAIL = 3
 DATASET_LOGLEVEL_ALL = 4
 
 
-class Custom_Dataset(object):
+class CustomDataset(object):
     """This is the base class for all (external) data sets in QAL-
     Note: The fields are named like this to not appear as parameters in automatic generators like sql_xml.
+    TODO: Field names should be considered captions, not references, and should not be used when looping columns.
     """
 
-    field_names = []
-    """These are, if applicable, the field names of the dataset"""
-    field_types = []
+    field_names = None
+    """These are, if applicable, the field names of the resulting dataset"""
+    field_types = None
     """The data types of the fields"""
-    data_table = [] 
+    data_table = None
     """The actual data, in the form of a two-dimensional array"""
+    _base_path = None
+    """The base path for all relative references in resources. Inferred from the locations of resource files."""
+
     
     _log_level = DATASET_LOGLEVEL_MEDIUM
 
@@ -37,15 +41,19 @@ class Custom_Dataset(object):
     def __init__(self):
         """Constructor""" 
         self.field_names = []
+        self.field_types = []
         self.data_table = []
+        self._base_path = None
         self._log = [] 
         self._log_level = DATASET_LOGLEVEL_MEDIUM
         
     def cast_text_to_type(self, _text, _field_idx):
         try:
-            if  self.field_types[_field_idx] == "integer":
+            if self.field_types is None:
+                return _text
+            elif self.field_types[_field_idx] == "integer":
                 return int(_text)
-            if  self.field_types[_field_idx] == "float":
+            elif self.field_types[_field_idx] == "float":
                 return float(_text)
             else:
                 return _text
@@ -55,7 +63,7 @@ class Custom_Dataset(object):
     def log_update_row(self, _row_key, _old_row, _new_row, _comment = None):
         if self._log_level >= DATASET_LOGLEVEL_DETAIL:
             _field_diffs = []
-            for _field_idx in range(len(_new_row)):
+            for _field_idx in range(len(self.field_names)):
                 if _old_row[_field_idx] != _new_row[_field_idx]:
                     _field_diffs.append(self.field_names[_field_idx] + " : " + quote(str(_old_row[_field_idx])) + " =>" + quote(str(_new_row[_field_idx]))) 
                 
@@ -77,31 +85,38 @@ class Custom_Dataset(object):
     def log_load(self, _filename, _comment = None):
         if self._log_level >= DATASET_LOGLEVEL_LOW:
             self._log.append(self.__class__.__name__ + ".load;"+quote(str(_filename)) + ";" + str(datetime.now().isoformat()) + empty_if_none(";"+ str(_comment), _comment))
-               
-        
+
+    def read_resource_settings(self, _resource):
+        """Load the data"""
+        raise Exception('CustomDataset.read_resource_settings is not implemented in class: ' + self.__class__.__name__)
+
+    def write_resource_settings(self, _resource):
+        """Load the data"""
+        raise Exception('CustomDataset.write_resource_settings is not implemented in class: ' + self.__class__.__name__)
+
     def load(self):
         """Load the data"""
-        raise Exception('Custom_Dataset.Load is not implemented in class: ' + self.classname)
+        raise Exception('CustomDataset.load is not implemented in class: ' + self.__class__.__name__)
         pass
     
     def save(self):
         """Save the data to the underlying target."""
-        raise Exception('Custom_Dataset.Save is not implemented in class: ' + self.classname)
+        raise Exception('CustomDataset.save is not implemented in class: ' + self.__class__.__name__)
 
     
-    def _structure_insert_row(self, _row_idx, _row_data, _no_logging = False):
+    def _structure_insert_row(self, _row_idx, _row_data, _commit=None,_no_logging = False):
         """Inserts a row at _row_idx in the self.data_table, containing _row_data\
         Commonly overridden by subclasses"""
         if _no_logging == False:
             self.log_insert(_row_idx, _row_data)
         self.data_table.insert(_row_idx,_row_data)
-    def _structure_update_row(self, _row_idx, _row_data, _no_logging = False):
+    def _structure_update_row(self, _row_idx, _row_data, _commit=None, _no_logging = False):
         """Updates the row at _row_idx in the self.data_table with _row_data.\
         Commonly overridden by subclasses"""
         if _no_logging == False:
             self.log_update_row(_row_idx, self.data_table[_row_idx], _row_data)
         self.data_table[_row_idx] = _row_data
-    def _structure_delete_row(self, _row_idx, _no_logging = False):
+    def _structure_delete_row(self, _row_idx, _commit=None, _no_logging = False):
         """Deletes a row at _row_idx from the self.data_table.\
         Commonly overridden by subclasses"""
         if _no_logging == False:
@@ -115,7 +130,7 @@ class Custom_Dataset(object):
         pass
 
         
-    def _structure_apply_merge(self, _insert, _update, _delete, _sorted_dest):
+    def _structure_apply_merge(self, _insert, _update, _delete, _sorted_dest, _commit=True):
         
         #print("_insert: " + str(_insert))
         #print("Before apply: " + str(_sorted_dest))
@@ -143,22 +158,22 @@ class Custom_Dataset(object):
   
             # Make deletes
             while _delete_idx > -1 and _delete[_delete_idx][1] == _curr_row_idx:
-                self._structure_delete_row(_curr_row_idx)
+                self._structure_delete_row(_curr_row_idx, _commit=_commit)
                 _delete_idx-= 1
             # Make updates
             while _update_idx > -1 and _update[_update_idx][1] == _curr_row_idx:
-                self._structure_update_row(_curr_row_idx, _update[_update_idx][2])
+                self._structure_update_row(_curr_row_idx, _update[_update_idx][2], _commit=_commit)
                 _update_idx-= 1
                 
             # Make inserts
             while _insert_idx > -1 and _insert[_insert_idx][1] == _curr_row_idx:
-                self._structure_insert_row(_curr_row_idx, _insert[_insert_idx][2]) 
+                self._structure_insert_row(_curr_row_idx, _insert[_insert_idx][2], _commit=_commit)
                 _insert_idx-= 1
         
         #print("After apply:  " + str(_sorted_dest))            
         return self.data_table        
     
-    def apply_new_data(self, _new_data_table, _key_fields):
+    def apply_new_data(self, _new_data_table, _key_fields, _insert=None, _update=None, _delete=None, _commit=True):
         """This function applies a new data table unto the existing, matches are made using the key fields.
       
         
@@ -176,17 +191,26 @@ class Custom_Dataset(object):
         self._structure_init()        
         
         # Compare the source and destination table to generate difference sets
-        _delete, _insert, _update, _dest_sorted = compare(
+        _deletes, _inserts, _updates, _dest_sorted = compare(
                                                           _left = _new_data_table, 
                                                           _right = self.data_table, 
                                                           _key_columns = _key_fields, 
                                                           _full = True)
         
         # Merge the data into the structure. 
-        # Note: in RDBMS_Dataset, this currently means writing to the underlying database, since there is no in-memory structure.
-        self.data_table = self._structure_apply_merge(_insert, _update, _delete, _dest_sorted)
+        # Note: in RDBMSDataset, this currently means writing to the underlying database, since there is no in-memory structure.
+
+        # Clear out all changes that should not be made
+        if _delete is None or _delete==False:
+            _deletes=None
+        if _insert is None or _insert==False:
+            _inserts=None
+        if _update is None or _update==False:
+            _updates=None
+
+        self.data_table = self._structure_apply_merge(_inserts, _updates, _deletes, _dest_sorted, _commit=_commit)
         
-        return self.data_table
+        return self.data_table, _deletes, _inserts, _updates
                             
                 
         
@@ -237,3 +261,4 @@ class Custom_Dataset(object):
             
                     
         return str("\nUNION\n".join(_result)) 
+
