@@ -5,19 +5,20 @@ Created on Sep 14, 2012
 '''
 
 import io
+from qal.common.strings import make_path_absolute
 
-from qal.dataset.custom import Custom_Dataset
+from qal.dataset.custom import CustomDataset
 from qal.common.listhelper import find_next_match, find_previous_match
 from lxml import _elementpath
 from lxml import etree
-from xml.etree.ElementTree import SubElement, _Element  
+from lxml.etree import SubElement
 
 
 
 def xpath_data_formats():
     return ["XML", "XHTML", "HTML"] # "UNTIDY_HTML"]
 
-class XPath_Dataset(Custom_Dataset):
+class XpathDataset(CustomDataset):
     """This class implements data formats that are possible to query via XPath.
     Currently XML and HTML are implemented(XHTML isn't tested). 
     Untidy HTML will be implemented using the Beautiful Soup library.
@@ -29,12 +30,11 @@ class XPath_Dataset(Custom_Dataset):
     rows_xpath = None
     """The XPath to the "row" nodes of the data set. Ex: /books/book"""
 
-    field_xpaths  = []
+    field_xpaths  = None
     """A list of XPaths relative to the rows_xpath leading to each field. Ex: ["title", "id"]"""
 
     xpath_data_format = None
     """The data format. Can be either "XML", "XHTML" or "HTML"."""
-    
 
     _structure_tree = None
     """A private reference to the node tree, when load is called with _add_node_ref=True, an instance
@@ -43,7 +43,7 @@ class XPath_Dataset(Custom_Dataset):
     _structure_row_node_parent = None
     """A private reference to the parent node of the row nodes in the structure """     
     
-    _structure_key_fields = []
+    _structure_key_fields = None
     """A private list of key fields."""
     
     _structure_row_node_name = None
@@ -53,8 +53,11 @@ class XPath_Dataset(Custom_Dataset):
         .. todo::
             Should this be able to hold an XML?
     """
-        
-    
+
+    _structure_top_node = None
+    """
+        A private variable holding the top node of the structure.
+    """
 
     def __init__(self, _filename = None, _rows_xpath = None,  _resource = None):
 
@@ -63,7 +66,10 @@ class XPath_Dataset(Custom_Dataset):
         :param str _rows_xpath: Path to nodes holding the rows.
         :param Resource _resource : An instance of a :py:class: qal.common.resource.resource
         """
-        super(XPath_Dataset, self ).__init__()
+        super(XpathDataset, self ).__init__()
+
+        self.field_xpaths = []
+        self._structure_key_fields = []
         
         if _resource:
             self.read_resource_settings(_resource)
@@ -75,23 +81,43 @@ class XPath_Dataset(Custom_Dataset):
             if _rows_xpath != None: 
                 self.rows_xpath = _rows_xpath
             else:
-                self.rows_xpath = None  
+                self.rows_xpath = None
+
+
+
                 
 
 
     def read_resource_settings(self, _resource):
         """Reads settings from a resource"""
+
+        self._base_path = _resource.base_path
   
         if _resource.type.upper() != 'XPATH':
-            raise Exception("XPath_Dataset.read_resource_settings.parse_resource error: Wrong resource type: " + _resource.type)
-        self.filename   =   _resource.data.get("filename")
-        self.rows_xpath =   _resource.data.get("rows_xpath")
+            raise Exception("XpathDataset.read_resource_settings.parse_resource error: Wrong resource type: " + _resource.type)
+        self.filename   = _resource.data.get("filename")
+        self.rows_xpath = _resource.data.get("rows_xpath")
         self.xpath_data_format =  _resource.data.get("xpath_data_format")
         self.field_names = _resource.data.get("field_names")
         self.field_xpaths = _resource.data.get("field_xpaths")       
         self.field_types = _resource.data.get("field_types") 
         self.xpath_text_qualifier = _resource.data.get("xpath_text_qualifier") 
-        self.encoding = _resource.data.get("encoding")  
+        self.encoding = _resource.data.get("encoding")
+
+
+    def write_resource_settings(self, _resource):
+        """Write settings to a resource"""
+
+        _resource.type = 'XPATH'
+        _resource.data.clear()
+        _resource.data["filename"] = self.filename
+        _resource.data["rows_xpath"] = self.rows_xpath
+        _resource.data["xpath_data_format"] = self.xpath_data_format
+        _resource.data["field_names"] = self.field_names
+        _resource.data["field_xpaths"] = self.field_xpaths
+        _resource.data["field_types"] = self.field_types
+        _resource.data["xpath_text_qualifier"] = self.xpath_text_qualifier
+        _resource.data["encoding"] = self.encoding
               
     def _file_to_tree(self, _data_format, _reference):
         """Reads a file and chooses the right parser to make it an lxml element tree"""
@@ -125,51 +151,57 @@ class XPath_Dataset(Custom_Dataset):
         print("Loading : " + self.filename)
         self.log_load(self.filename)  
         try:
-            _tree = self._file_to_tree(self.xpath_data_format, self.filename)
+            _tree = self._file_to_tree(self.xpath_data_format, make_path_absolute(self.filename, self._base_path))
             
         except Exception as e:
-            raise Exception("XPath_Dataset.load - error parsing " + self.xpath_data_format + " file : " + str(e))
+            raise Exception("XpathDataset.load - error parsing " + self.xpath_data_format + " file : " + str(e))
         
         #_root_nodes = _tree.xpath("/html/body/form/table/tr[4]/td[2]/table/tr[2]/td/table[2]/tr/td/table/tr[10]/td/table/tr")
         _root_nodes = _tree.xpath(self.rows_xpath)
-        if len(_root_nodes) > 0 and _add_node_ref:
+        if len(_root_nodes) > 0:
             self._structure_tree = _tree
             self._structure_row_node_parent = _root_nodes[0].getparent()
             self._structure_top_node = _tree.getroot()
         
         _data = []
-        for _curr_row in _root_nodes:
-            _row_data = []
-            for _field_idx in range(0, len(self.field_names)):
-                _curr_path, _curr_attribute = self._structure_parse_qal_xpath(self.field_xpaths[_field_idx])
-                if _curr_path != "":
-                    _item_data = _curr_row.xpath(_curr_path)[0]
-                else:
-                    _item_data = _curr_row
-                    
-                if _item_data is not None:
-                    if _curr_attribute:
-                        if _curr_attribute[0] == "@":
-                            _row_data.append(self.cast_text_to_type(_item_data.get(_curr_attribute[1:]), _field_idx))
-                        elif _curr_attribute.lower() == "xml":
-                            _row_data.append(self.cast_text_to_type(_item_data.tostring(), _field_idx))
+
+        if self.field_xpaths is not None and len(self.field_xpaths) > 0:
+            for _curr_row in _root_nodes:
+                _row_data = []
+                for _field_idx in range(0, len(self.field_names)):
+                    _curr_path, _curr_attribute = self._structure_parse_qal_xpath(self.field_xpaths[_field_idx])
+                    if _curr_path != "":
+                        _found_node = _curr_row.xpath(_curr_path)
+                        if len(_found_node)>0:
+                            _item_data = _found_node[0]
                         else:
-                            raise Exception("Param_XPath.load: Error, attribute references must have the ::@AttributeName- format or ::XML. The attribute was \""+_curr_attribute + "\"")
+                            _item_data = None
                     else:
-                        _row_data.append(self.cast_text_to_type(_item_data.text, _field_idx))
-                else:
-                    _row_data.append("")
+                        _item_data = _curr_row
+
+                    if _item_data is not None:
+                        if _curr_attribute:
+                            if _curr_attribute[0] == "@":
+                                _row_data.append(self.cast_text_to_type(_item_data.get(_curr_attribute[1:]), _field_idx))
+                            elif _curr_attribute.lower() == "xml":
+                                _row_data.append(self.cast_text_to_type(_item_data.tostring(), _field_idx))
+                            else:
+                                raise Exception("Param_XPath.load: Error, attribute references must have the ::@AttributeName- format or ::XML. The attribute was \""+_curr_attribute + "\"")
+                        else:
+                            _row_data.append(self.cast_text_to_type(_item_data.text, _field_idx))
+                    else:
+                        _row_data.append("")
 
 
-            # Add reference to XML structure.
-            if _add_node_ref:
-                _row_data.append(_curr_row)
-            
-            print(str(_row_data))
-            _data.append(_row_data)
-            
-        self.data_table = _data
-        
+                # Add reference to XML structure.
+                if _add_node_ref:
+                    _row_data.append(_curr_row)
+
+                print(str(_row_data))
+                _data.append(_row_data)
+
+            self.data_table = _data
+
         return _data  
     
     
@@ -240,6 +272,8 @@ class XPath_Dataset(Custom_Dataset):
                         
                     _token_idx+=5
                 else:
+                    if isinstance(_curr_node, str):
+                        raise Exception("_structure_create_xpath_nodes - Internal error: Node is a string, missing node ref, was _add_node_ref used? \nData :" + _curr_node+" | "+_next_name+" | "+_xpath+" | "+str(_tokens))
                     _found_nodes = _curr_node.xpath(_next_name)
                     # Node found, move on
                     if len(_found_nodes) == 1:
@@ -300,23 +334,23 @@ class XPath_Dataset(Custom_Dataset):
     
     def _structure_init(self):
         """Initializes the XML structure that data is to be applied to."""
-        print("XPath_Dataset._structure_init")
-        super(XPath_Dataset, self)._structure_init()
+        print("XpathDataset._structure_init")
+        super(XpathDataset, self)._structure_init()
 
         # Parse important information data from XPath 
         _root_node_name, self._structure_row_node_name, _parent_xpath = self._structure_parse_root_path(self.rows_xpath)
         
         # If the structure already loaded?
-        if not self._structure_row_node_parent:
+        if self._structure_row_node_parent is None:
             
             # If not try to load, or create file.    
             import os
-            if os.path.exists(self.filename):
+            if os.path.exists(make_path_absolute(self.filename, self._base_path)):
                 
                 try:
                     self.load(_add_node_ref=True)
                 except Exception as e:
-                    raise Exception("XPath_Dataset.save - error parsing " + self.xpath_data_format + " file : " + str(e))
+                    raise Exception("XpathDataset.save - error parsing " + self.xpath_data_format + " file : " + str(e))
             else:
                 # Create a tree with root node based on the first  
                 
@@ -328,7 +362,7 @@ class XPath_Dataset(Custom_Dataset):
                               
                     _tree = etree.parse(io.StringIO("<?xml version='1.0' ?>\n<" + _root_node_name + "/>")) 
                 else:
-                    raise Exception("XPath_Dataset.save - rows_xpath("+ str(self.rows_xpath)+") must be absolute and have at least the name of the root node. Example: \"/root_node\" ")
+                    raise Exception("XpathDataset.save - rows_xpath("+ str(self.rows_xpath)+") must be absolute and have at least the name of the root node. Example: \"/root_node\" ")
 
         # If the structure there yet? It could be an XML file with only a top node. 
         if self._structure_row_node_parent is None:
@@ -341,7 +375,7 @@ class XPath_Dataset(Custom_Dataset):
        
         
     def _structure_populate_row(self, _node, _row_data):
-        for _field_idx in range(len(_row_data)):
+        for _field_idx in range(len(self.field_xpaths)):
             _curr_path, _curr_attribute = self._structure_parse_qal_xpath(self.field_xpaths[_field_idx])
             _curr_node = self._structure_create_xpath_nodes(_node, _curr_path)
             _new_value = str(_row_data[_field_idx])
@@ -353,52 +387,55 @@ class XPath_Dataset(Custom_Dataset):
             if _curr_attribute:
                 _curr_node.set(_curr_attribute[1:], _new_value)
             else:
-                _curr_node.text = _new_value 
-            
+                _curr_node.text = _new_value
+
              
-    def _structure_insert_row(self, _row_idx, _row_data):
-        """Override parent to add XML handling"""
+    def _structure_insert_row(self, _row_idx, _row_data, _commit=True, _no_logging=None):
+        if _commit is True:
+            """Override parent to add XML handling"""
 
-        # Last column is a reference
-        print("_row_data: " + str(_row_data))
-        _row_node = self.data_table[_row_idx][len(self.data_table[_row_idx]) -1]
-        
-        _new_element = etree.Element(self._structure_row_node_name)
-        
-        # Insert row node
-        self._structure_row_node_parent.insert(_row_idx, _new_element)
-        
-        self._structure_populate_row(_new_element, _row_data)
+            # Last column is a reference
+            print("_row_data: " + str(_row_data))
+            _row_node = self.data_table[_row_idx][len(self.data_table[_row_idx]) -1]
+
+            _new_element = etree.Element(self._structure_row_node_name)
+
+            # Insert row node
+            self._structure_row_node_parent.insert(_row_idx, _new_element)
+
+            self._structure_populate_row(_new_element, _row_data)
 
         # Call parent
-        super(XPath_Dataset, self)._structure_insert_row(_row_idx,_row_data)
+        super(XpathDataset, self)._structure_insert_row(_row_idx,_row_data)
         
-    def _structure_update_row(self, _row_idx, _row_data):
-        """Override parent to add XML handling"""
+    def _structure_update_row(self, _row_idx, _row_data, _commit=True, _no_logging=None):
+        if _commit is True:
+            """Override parent to add XML handling"""
 
-        # Last column is a reference
-        print("_row_data: " + str(_row_data))
-        _row_node = self.data_table[_row_idx][len(self.data_table[_row_idx]) -1]
-        
-       
-        self._structure_populate_row(_row_node, _row_data)
+            # Last column is a reference
+            print("_row_data: " + str(_row_data))
+            _row_node = self.data_table[_row_idx][len(self.data_table[_row_idx]) -1]
+
+
+            self._structure_populate_row(_row_node, _row_data)
         
         # Call parent
-        super(XPath_Dataset, self)._structure_update_row(_row_idx,_row_data)
+        super(XpathDataset, self)._structure_update_row(_row_idx,_row_data)
 
-    def _structure_delete_row(self, _row_idx):
-        """Override parent to add XML handling"""
-        _row_node = self.data_table[_row_idx][len(self.data_table[_row_idx]) -1]
-        self._structure_row_node_parent.remove(_row_node)
+    def _structure_delete_row(self, _row_idx, _commit=True, _no_logging=None):
+        if _commit is True:
+            """Override parent to add XML handling"""
+            _row_node = self.data_table[_row_idx][len(self.data_table[_row_idx]) -1]
+            self._structure_row_node_parent.remove(_row_node)
         # Call parent
-        super(XPath_Dataset, self)._structure_delete_row(_row_idx)
+        super(XpathDataset, self)._structure_delete_row(_row_idx)
         #self.data_table.pop(_row_idx)
 
     def save(self, _save_as = None):
         """Save the document"""
         
         if not _save_as:
-            _save_as = self.filename
+            _save_as = make_path_absolute(self.filename, self._base_path())
             
         self._structure_tree.write(_save_as)
             
